@@ -24,6 +24,7 @@ import json
 import re
 from typing import Optional, List
 from dataclasses import asdict
+import xml.dom.minidom
 import requests
 
 from chatie_grpc.wechaty import (  # type: ignore
@@ -158,7 +159,7 @@ class HostiePuppet(Puppet):
     """
 
     def __init__(self, options: PuppetOptions, name: str = 'hostie_puppet'):
-        super(HostiePuppet, self).__init__(options, name)
+        super().__init__(options, name)
 
         if options.token is None:
             if WECHATY_PUPPET_HOSTIE_TOKEN is None:
@@ -195,7 +196,7 @@ class HostiePuppet(Puppet):
             raise WechatyPuppetGrpcError('can"t get room_list response')
         return response.ids
 
-    async def message_image(self, message_id: str, image_type: ImageType
+    async def message_image(self, message_id: str, image_type: ImageType = 3
                             ) -> FileBox:
         """
         get message image data
@@ -203,14 +204,15 @@ class HostiePuppet(Puppet):
         :param image_type:
         :return:
         """
-        response = await self.puppet_stub.message_image(
-            id=message_id,
-            type=image_type)
-        if response is None:
-            # TODO -> need to refactor the raised error
-            raise WechatyPuppetGrpcError('response is invalid')
+        response = await self.puppet_stub.message_image(id=message_id, type=image_type)
         json_response = json.loads(response.filebox)
-        return FileBox.from_json(obj=json_response)
+        if 'base64' not in json_response:
+            raise WechatyPuppetGrpcError('image response data structure is not correct')
+        file_box = FileBox.from_base64(
+            json_response['base64'],
+            name=json_response['name']
+        )
+        return file_box
 
     def on(self, event_name: str, caller):
         """
@@ -418,10 +420,15 @@ class HostiePuppet(Puppet):
         elif payload.type == MessageType.MESSAGE_TYPE_MINI_PROGRAM:
             mini_program = await self.message_mini_program(message_id=message_id)
             await self.message_send_mini_program(conversation_id=to_id, mini_program=mini_program)
-        # TODO
-        # elif payload.type == MessageType.MESSAGE_TYPE_EMOTICON:
-        # elif payload.type == MessageType.MESSAGE_TYPE_AUDIO:
+        elif payload.type == MessageType.MESSAGE_TYPE_EMOTICON:
+            file_box = await self.message_emoticon(message=payload.text)
+            await self.message_send_file(conversation_id=to_id, file=file_box)
+        elif payload.type == MessageType.MESSAGE_TYPE_AUDIO:
+            raise WechatyPuppetOperationError('Can not support audio message forward')
         # elif payload.type == MessageType.ChatHistory:
+        elif payload.type == MessageType.MESSAGE_TYPE_IMAGE:
+            file_box = await self.message_image(message_id=message_id, image_type=3)
+            await self.message_send_file(conversation_id=to_id, file=file_box)
         else:
             file_box = await self.message_file(message_id=message_id)
             await self.message_send_file(conversation_id=to_id, file=file_box)
@@ -433,12 +440,20 @@ class HostiePuppet(Puppet):
         :return:
         """
         response = await self.puppet_stub.message_file(id=message_id)
-        json_response = json.loads(response.filebox)
-        if 'base64' not in json_response:
-            raise WechatyPuppetGrpcError('file response data structure is not correct')
-        file_box = FileBox.from_base64(
-            json_response['base64'],
-            name=json_response['name']
+        file_box = FileBox.from_json(response.filebox)
+        return file_box
+
+    async def message_emoticon(self, message: str) -> FileBox:
+        """
+        emoticon from message
+        :param message:
+        :return:
+        """
+        DOMTree = xml.dom.minidom.parseString(message)
+        collection = DOMTree.documentElement
+        file_box = FileBox.from_url(
+            url=collection.getElementsByTagName('emoji')[0].getAttribute('cdnurl'),
+            name=collection.getElementsByTagName('emoji')[0].getAttribute('md5') + '.gif'
         )
         return file_box
 
@@ -474,8 +489,9 @@ class HostiePuppet(Puppet):
         response_dict = json.loads(response.mini_program)
         try:
             mini_program = MiniProgramPayload(**response_dict)
-        except Exception:
-            raise ValueError(f'can"t init mini-program payload {response_dict}')
+        except Exception as e:
+            raise WechatyPuppetPayloadError(f'can"t init mini-program payload {response_dict}')\
+                from e
         return mini_program
 
     async def contact_alias(self, contact_id: str, alias: Optional[str] = None
@@ -964,7 +980,7 @@ class HostiePuppet(Puppet):
                         invited_ids=payload_data.get('inviteeIdList', []),
                         inviter_id=payload_data.get('inviterId'),
                         room_id=payload_data.get('roomId'),
-                        time_stamp=payload_data.get('timestamp')
+                        timestamp=payload_data.get('timestamp')
                     )
                     self._event_stream.emit('room-join', payload)
 
@@ -981,8 +997,8 @@ class HostiePuppet(Puppet):
                     payload = EventRoomLeavePayload(
                         removed_ids=payload_data.get('removeeIdList', []),
                         remover_id=payload_data.get('removerId'),
-                        room_id=payload_data.get('removerId'),
-                        time_stamp=payload_data.get('timestamp')
+                        room_id=payload_data.get('roomId'),
+                        timestamp=payload_data.get('timestamp')
                     )
                     self._event_stream.emit('room-leave', payload)
 
@@ -993,7 +1009,7 @@ class HostiePuppet(Puppet):
                         new_topic=payload_data.get('newTopic'),
                         old_topic=payload_data.get('oldTopic'),
                         room_id=payload_data.get('roomId'),
-                        time_stamp=payload_data.get('timestamp')
+                        timestamp=payload_data.get('timestamp')
                     )
                     self._event_stream.emit('room-topic', payload)
 
